--***************************************************************
--* File         : WBRAM.vhd                                    *
--* Created      : 2009/11/23 12:07:38 Osamu Kawashima          *
--* Modified     : Time-stamp: <09/11/23 18:19:41 kawasan>      *
--*                                                             *
--* WISHBONE RAM (Type Xilinx BlockRAM)                         *
--*                                                             *
--* Copyright 2009 Osamu Kawashima. all rights reserved         *
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity WBRAM is
	generic (
		ADDRESS_WIDTH	: integer := 10;	-- Spartan2(4096bit/BRAM=8) Spartan3(18432bit/BRAM=10)
		DATA_WIDTH      : integer := 16
	);
	port (
		-- WISHBONE SLAVE interface:
		WB_CLK_I        : in    std_logic;
		WB_RST_I        : in    std_logic;
		WB_ADR_I        : in    std_logic_vector(31 downto 0);
		WB_DAT_I        : in    std_logic_vector(31 downto 0);
		WB_WE_I         : in    std_logic;
		WB_STB_I        : in    std_logic;
		WB_ACK_O        : out   std_logic;
		WB_DAT_O        : out   std_logic_vector(31 downto 0)
	);
end WBRAM;

architecture RTL of WBRAM is

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');

	type ram_type is array (0 to 2**ADDRESS_WIDTH-1) of std_logic_vector (DATA_WIDTH-1 downto 0);
	signal ram              : ram_type;

	signal addr_tmp         : std_logic_vector(ADDRESS_WIDTH-1 downto 0);
	signal addr             : std_logic_vector(ADDRESS_WIDTH-1 downto 0);
	signal wr_en            : std_logic;
	signal rd_en            : std_logic;
	signal ack_is           : std_logic;

begin

	wr_en <= '1' when WB_STB_I='1' and WB_WE_I='1' else '0';

	rd_en <= '1' when WB_STB_I='1' and WB_WE_I='0' else '0';

	addr_tmp <= WB_ADR_I(ADDRESS_WIDTH-1 downto 0);

	process (WB_CLK_I)
	begin
		if WB_CLK_I'event and WB_CLK_I='1' then
			if wr_en='1' then
				ram(CONV_INTEGER(addr_tmp)) <= WB_DAT_I(DATA_WIDTH-1 downto 0);
			end if;
			addr <= addr_tmp;
		end if;
	end process;

	WB_DAT_O <= ZERO32(31 downto DATA_WIDTH) & ram(CONV_INTEGER(addr));

	process (WB_RST_I, WB_CLK_I)
	begin
		if WB_RST_I='1' then
			ack_is <= '0';
		elsif WB_CLK_I'event and WB_CLK_I='1' then
			if WB_STB_I='0' then
				ack_is <= '0';
			elsif rd_en='1' then
				ack_is <= '1';
			else
				ack_is <= ack_is;
			end if;
		end if;
	end process;

	WB_ACK_O <= '0' when WB_STB_I='0' else '1' when wr_en='1' else ack_is;

end RTL;
